/**--------------------------------------------------------------------------
 **
 **        NEC PC-8001 Emulator "N80" for MacOS X
 **        --------------------------------------------
 **        Copyright (c) 1993-2012 H.Kanemoto2
 **
 **        header file
 **
 **        Rev        Date        Descriptions
 **        -----    ----------    -----------------------------------------------
 **        1.00    2010/12/08    Zaurus SL-C3000 Fedora Core 10 ARMî≈ porting
 **        1.10    2012/08/22    MacOS X 10.7(Lion) or later porting
 **------------------------------------------------------------------------*/
#ifndef _N80_H_
#define _N80_H_
#include "n80types.h"
#include "z80.h"
/**--------------------------------------------------------------------------
 **	define key codes
 **------------------------------------------------------------------------*/
#define VK_TAB		(0x0F)
#define VK_ESCAPE	(0x01)
#define VK_BACK		(0x0E)
#define VK_INSERT	(0x6E)
#define VK_DELETE	(0x0E)
#define VK_RETURN	(0x1C)
#define VK_SPACE	(0x39)
#define VK_UP		(0x67)
#define VK_DOWN		(0x6C)
#define VK_LEFT		(0x69)
#define VK_RIGHT	(0x6A)
#define VK_F6		(0x40)
#define VK_F7		(0x41)
#define VK_F8		(0x42)
#define VK_F9		(0x43)
#define VK_F10		(0x44)
#define VK_F12		(0x58)
#define VK_END		(0x6B)
/**--------------------------------------------------------------------------
 **		PROTOTYPE DECLARATIONs
 **------------------------------------------------------------------------*/
struct tagCFGITEM;
typedef struct tagCFGITEM	CFGITEM, *LPCFGITEM;
struct tagCFGPAGE;
typedef struct tagCFGPAGE	CFGPAGE, *LPCFGPAGE;
struct CONFIG_tag;
typedef struct CONFIG_tag	CONFIG, *LPCONFIG;
struct CHOOSER_tag;
typedef struct CHOOSER_tag	CHOOSER, *LPCHOOSER;
struct INI_tag;
typedef struct INI_tag		INI, *PINI;
struct N80LX_tag;
typedef struct N80LX_tag	N80LX, *LPN80LX;
/**--------------------------------------------------------------------------
 **    declare structure types
 **------------------------------------------------------------------------*/
typedef struct tagKEYEVT {
	USHORT		key;
    BYTE		code;	// PC-8001 Key Code
	BOOL		SHIFT;
	BOOL		ALT;
	BOOL		CTL;
    char		ascii;
}   KEYEVTG, *LPKEYEVT;

struct tagCFGITEM {
	int			id;
	int			type;
	char*		name;
	char*		hint;
	int			value;
	int			(*controller)(LPCFGITEM, int, LPVOID);
	LPVOID		arg;
	LPCFGPAGE	parent;
};

struct tagCFGPAGE {
	LPCONFIG	parent;
	int			page;
	LPVOID		arg;
	LPCFGITEM	items;
	int			item_cnt;
	int			cur_idx;
	LPCFGITEM	(*get_item)(LPCFGPAGE);
	int			(*handler)(LPCFGPAGE, LPKEYEVT);
	void		(*prepare)(LPCFGPAGE, LPVOID arg);
	int			(*call)(LPCFGPAGE, int, LPVOID);
	void		(*reflesh)(LPCFGPAGE);
	int			jmp_page;
	LPVOID		jmp_arg;
};

struct CONFIG_tag {
	LPVOID		parent;
	char		text[25][80];
	char		attr[25][80];
	int			fd;
	KEYEVTG		kv;
	CFGPAGE		pages[2];
	int			page;
	int			maxpages;
	void		(*init)(LPCONFIG, LPVOID);
	void		(*open)(LPCONFIG);
	void		(*init_page)(LPCONFIG, int page, LPVOID arg);
	BOOL		(*dispatch)(LPCONFIG, LPKEYEVT);
	void		(*drawitem)(LPCFGITEM item, int line, BOOL selected);
	void		(*show)(LPCONFIG);
};

typedef struct N80LX_tag {
    DWORD                   dot_npattern[8][256][8];    // narrow(width80) dot patterns [color][bit pattern][horizon]
    DWORD                   dot_wpattern[8][256][16];   // wide(width40) dot patterns [color][bit pattern][horizon]
    volatile int32_t        vram_sync;                  // PC-8001 vram sync
    volatile int32_t        request_cpu_signal;         // signal for z80 thread
    volatile DWORD          request_alsa_signal;        // signal for alsa thread
    volatile int32_t        request_disp_signal;        // signal for display thread
    int                     fn_on;                      // zaurus [Fn] key press status
    int                     sampling_rate;              // alsa pcm sampling rate
    int                     period_frames;              // alsa period length in samples
    int                     period_bytes;               // alsa period length in byte
    LPUSHORT                period_buffer;              // alsa local buffer
    int                     period_usec;                // alsa period long in micor sec
    double                  frame_keisu;                // 44.1khz(PCM) / 4MHz(CPU)
    int                     phase_adjuster;             // wave phase adjust (beep)
    int                     phase_adjuster2;            // wave phase adjust (pcg sound)
    BYTE                    cmt_buff[32*1024];          // CMT File cache
	CONFIG					config;						// config
	char					save_cmt_name[64];
	char					*n80rom_filename;
	char					*_4throm_filename;
	char					*fdurom_filename;

	DWORD					tmp_save_screen_mode;
	BYTE					tmp_save_text_visible;
	BOOL					tmp_save_use_new_pcg;
	BOOL					tenkeyless;
	BOOL					nowait;

}   N80LX, *LPN80LX;

enum N80COLOR {N80BLACK = 0, N80BLUE, N80RED, N80PERPLE, N80GREEN, N80CYAN, N80YELLOW, N80WHITE};
#endif//_N80_H_
